@@ -82,7 +82,7 @@ impl RnpStatus {
     /// Returns the [`RnpResult`] suitable as return value for RNP
     /// functions.
     pub fn epilogue(&self, name: &str, args: Vec<String>) -> RnpResult {
-        if ! self.success() || call_tracing() {
+        if ! self.success() || call_tracing() || full_tracing() {
             if cfg!(debug_assertions) {
                 log!("sequoia-octopus: TRACE: {}({}) => {}",
                      name, args.join(", "), self)
@@ -265,7 +265,9 @@ pub type Result<T> = std::result::Result<T, RnpStatus>;
 
 //#[cfg(windows)]
 pub fn log_internal<T: AsRef<str>>(text: T) {
-    let text = format!("{}: {}", chrono::offset::Utc::now(), text.as_ref());
+    let text = format!("{}: {}",
+                       chrono::offset::Utc::now().format("%T"),
+                       text.as_ref());
 
     if cfg!(windows) {
         // Save messages to a log file in the current profile's
@@ -496,8 +498,8 @@ macro_rules! rnp_function {
         macro_rules! assert_ptr_int {
             ( $param: expr ) => {
                 if $param.is_null() {
-                    warn!("parameter {:?} is null", stringify!($param));
-                    rnp_return_status!(RNP_ERROR_NULL_POINTER);
+                    warn!("parameter {:?} is NULL", stringify!($param));
+                    rnp_return_status!(crate::error::RNP_ERROR_NULL_POINTER);
                 }
             };
         }
@@ -548,7 +550,7 @@ macro_rules! rnp_function {
                     Err(e) => {
                         warn!("parameter {:?} is not UTF8: {}",
                               stringify!($param), e);
-                        rnp_return_status!(RNP_ERROR_BAD_PARAMETERS);
+                        rnp_return_status!(crate::error::RNP_ERROR_BAD_PARAMETERS);
                     }
                 }
             }};
@@ -570,7 +572,7 @@ macro_rules! rnp_function {
                 match $result {
                     Ok(v) => v,
                     Err(_) => {
-                        let s: RnpStatus = $fail_with;
+                        let s: crate::error::RnpStatus = $fail_with;
                         rnp_return_status!(s);
                     },
                 }
@@ -585,11 +587,11 @@ macro_rules! rnp_function {
                     if let Ok(e) = e.downcast::<crate::error::Error>() {
                         e.into()
                     } else {
-                        RNP_ERROR_GENERIC
+                        crate::error::RNP_ERROR_GENERIC
                     }
                 } else {
                     t!("Leaving function: success");
-                    RNP_SUCCESS
+                    crate::error::RNP_SUCCESS
                 })
             };
         }
@@ -604,6 +606,10 @@ macro_rules! rnp_function {
                 crate::THUNDERBIRD_WORKAROUND
             }};
         }
+
+        if crate::error::full_tracing() {
+            t!("Entering function");
+        }
     };
 }
 
