@@ -61,6 +61,7 @@ use crate::{
     conversions::*,
     error::*,
     signature::ca_signatures,
+    tbprofile::TBProfile,
 };
 
 pub struct RnpKey {
@@ -465,7 +466,14 @@ fn rnp_key_have_secret(key: *const RnpKey,
     let result = assert_ptr_mut!(result);
 
     let mut has_secret = (*key).has_secret();
-    if ! has_secret {
+
+    // If there is no secret in the Thunderbird keyring, we are
+    // looking at the gpg-agent store.  However, we have to be careful
+    // to only report secret keys after Thunderbird has created their
+    // encrypted passphrase file, otherwise Thunderbird will complain
+    // about the OpenPGP state being corrupted, and disable OpenPGP
+    // support.
+    if ! has_secret && TBProfile::has_encrypted_passphrase_txt() {
         has_secret = (*(*key).ctx).certs.key_on_agent(&(*key).fingerprint());
     }
 
@@ -1820,11 +1828,11 @@ fn rnp_key_remove_signatures(key: *mut RnpKey,
         // The primary key and related signatures.
         let pk_bundle = cert.primary_key().bundle();
         acc.push(pk_bundle.key().clone().into());
-        for s in pk_bundle.self_signatures() { acc.push(s.clone().into()); }
-        for s in pk_bundle.self_revocations()  { acc.push(s.clone().into()); }
+        for s in pk_bundle.self_signatures2() { acc.push(s.clone().into()); }
+        for s in pk_bundle.self_revocations2()  { acc.push(s.clone().into()); }
         if ! remove_non_selfsig {
-            for s in pk_bundle.other_revocations() { acc.push(s.clone().into()); }
-            for s in pk_bundle.certifications() { acc.push(s.clone().into()); }
+            for s in pk_bundle.other_revocations2() { acc.push(s.clone().into()); }
+            for s in pk_bundle.certifications2() { acc.push(s.clone().into()); }
         }
 
         // The subkeys and related signatures.
