@@ -948,10 +948,10 @@ impl KeystoreData {
         let now = SystemTime::now();
         let elapsed = now.duration_since(self.keys_on_agent_last_refresh);
         let expired = match elapsed {
-            // If the cache is more than 10 seconds out of date,
+            // If the cache is more than 1 minute out of date,
             // refresh it.
             Ok(elapsed) => {
-                if elapsed > Duration::new(10, 0) {
+                if elapsed > Duration::new(60, 0) {
                     true
                 } else {
                     false
@@ -1676,7 +1676,7 @@ mod tests {
             })?;
         }
 
-        let mut sig = builder.sign_subkey_binding(&mut pk_signer, pk, &key)?;
+        let sig = builder.sign_subkey_binding(&mut pk_signer, pk, &key)?;
 
         // Verify it.
         if let Err(err) = sig.verify_subkey_binding(pk_signer.public(), pk, &key) {
