@@ -37,7 +37,7 @@ use openpgp::{
 use openpgp::crypto::{self, SessionKey};
 use openpgp::packet::{PKESK, SKESK};
 
-use sequoia_ipc as ipc;
+use sequoia_gpg_agent::gnupg;
 
 use crate::{
     RnpContext,
@@ -52,6 +52,9 @@ use crate::{
     error::*,
 };
 
+/// Controls tracing in this module.
+const TRACE: bool = crate::TRACE;
+
 pub struct RnpOpVerify<'a> {
     ctx: &'a mut RnpContext,
     input: &'a mut RnpInput,
@@ -116,7 +119,7 @@ impl RnpOpVerifyResult {
     }
 
     /// Returns an verification failure (arbitrarily picks the first).
-    fn verification_error(&self) -> Option<RnpResult> {
+    fn verification_error(&self) -> Option<RnpStatus> {
         for s in &self.signatures {
             if s.status != RNP_SUCCESS {
                 return Some(s.status);
@@ -128,7 +131,7 @@ impl RnpOpVerifyResult {
 
 pub struct RnpOpVerifySignature {
     ctx: *mut RnpContext,
-    status: RnpResult,
+    status: RnpStatus,
     sig: Signature,
     key: Option<(Key<UnspecifiedParts, UnspecifiedRole>, Cert)>,
 }
@@ -182,18 +185,18 @@ fn rnp_op_verify_create<'a>(op: *mut *mut RnpOpVerify<'a>,
                             -> RnpResult
 {
     rnp_function!(rnp_op_verify_create, crate::TRACE);
-    assert_ptr!(op);
-    assert_ptr!(ctx);
-    assert_ptr!(input);
-    assert_ptr!(output);
+    let op = assert_ptr_mut!(op);
+    let ctx = assert_ptr_mut!(ctx);
+    let input = assert_ptr_mut!(input);
+    let output = assert_ptr_mut!(output);
 
     *op = Box::into_raw(Box::new(RnpOpVerify {
-        ctx: &mut *ctx,
-        input: &mut *input,
-        mode: SignatureMode::Inline(&mut *output),
+        ctx,
+        input,
+        mode: SignatureMode::Inline(output),
         result: Default::default(),
     }));
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
@@ -204,26 +207,29 @@ fn rnp_op_verify_detached_create<'a>(op: *mut *mut RnpOpVerify<'a>,
                                      -> RnpResult
 {
     rnp_function!(rnp_op_verify_detached_create, crate::TRACE);
-    assert_ptr!(op);
-    assert_ptr!(ctx);
-    assert_ptr!(input);
-    assert_ptr!(signature);
+    let op = assert_ptr_mut!(op);
+    let ctx = assert_ptr_mut!(ctx);
+    let input = assert_ptr_mut!(input);
+    let signature = assert_ptr_mut!(signature);
 
     *op = Box::into_raw(Box::new(RnpOpVerify {
-        ctx: &mut *ctx,
-        input: &mut *input,
-        mode: SignatureMode::Detached(&mut *signature),
+        ctx,
+        input,
+        mode: SignatureMode::Detached(signature),
         result: Default::default(),
     }));
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
 fn rnp_op_verify_destroy(op: *mut RnpOpVerify) -> RnpResult {
+    rnp_function!(rnp_op_verify_destroy, crate::TRACE);
+    arg!(op);
+
     if ! op.is_null() {
         drop(Box::from_raw(op));
     }
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
@@ -283,7 +289,7 @@ fn rnp_op_verify_execute(op: *mut RnpOpVerify) -> RnpResult {
         Ok(())
     }
 
-    if let Err(e) = f(op) {
+    rnp_return_status!(if let Err(e) = f(op) {
         if op.result.encrypted_message()
             && ! op.result.decrypted_successfully()
         {
@@ -303,7 +309,7 @@ fn rnp_op_verify_execute(op: *mut RnpOpVerify) -> RnpResult {
         }
     } else {
         RNP_SUCCESS
-    }
+    })
 }
 
 #[no_mangle] pub unsafe extern "C"
@@ -313,14 +319,15 @@ fn rnp_op_verify_get_signature_at(op: *mut RnpOpVerify,
                                   -> RnpResult {
     rnp_function!(rnp_op_verify_get_signature_at, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(signature);
+    arg!(idx);
+    let signature = assert_ptr_mut!(signature);
 
-    if let Some(s) = op.result.signatures.get(idx) {
+    rnp_return_status!(if let Some(s) = op.result.signatures.get(idx) {
         *signature = s as *const _;
         RNP_SUCCESS
     } else {
         RNP_ERROR_BAD_PARAMETERS
-    }
+    })
 }
 
 #[no_mangle] pub unsafe extern "C"
@@ -329,59 +336,63 @@ fn rnp_op_verify_get_signature_count(op: *mut RnpOpVerify,
                                      -> RnpResult {
     rnp_function!(rnp_op_verify_get_signature_count, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(count);
+    let count = assert_ptr_mut!(count);
     *count = op.result.signatures.len();
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_signature_get_handle(sig: *mut RnpOpVerifySignature,
+fn rnp_op_verify_signature_get_handle(sig: *const RnpOpVerifySignature,
                                       handle: *mut *mut RnpSignature)
                                       -> RnpResult {
     rnp_function!(rnp_op_verify_signature_get_handle, crate::TRACE);
-    assert_ptr!(sig);
-    assert_ptr!(handle);
-    let sig = &*sig;
+    let sig = assert_ptr_ref!(sig);
+    let handle = assert_ptr_mut!(handle);
+
     *handle =
         Box::into_raw(Box::new(
             RnpSignature::new(sig.ctx, sig.sig.clone(),
                               Some(sig.status == RNP_SUCCESS))));
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_signature_get_status(sig: *mut RnpOpVerifySignature)
+fn rnp_op_verify_signature_get_status(sig: *const RnpOpVerifySignature)
                                       -> RnpResult {
     rnp_function!(rnp_op_verify_signature_get_status, crate::TRACE);
     assert_ptr!(sig);
-    (*sig).status
+    rnp_return_status!((*sig).status)
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_signature_get_key(sig: *mut RnpOpVerifySignature,
+fn rnp_op_verify_signature_get_key(sig: *const RnpOpVerifySignature,
                                    key_out: *mut *mut RnpKey)
                                    -> RnpResult {
     rnp_function!(rnp_op_verify_signature_get_key, crate::TRACE);
-    assert_ptr!(sig);
-    assert_ptr!(key_out);
-    if let Some(key) = (*sig).key.as_ref().cloned() {
+    let sig = assert_ptr_ref!(sig);
+    let key_out = assert_ptr_mut!(key_out);
+
+    rnp_return_status!(if let Some(key) = (*sig).key.as_ref().cloned() {
         *key_out =
             Box::into_raw(Box::new(RnpKey::new((*sig).ctx, key.0, &key.1)));
         RNP_SUCCESS
     } else {
         *key_out = std::ptr::null_mut();
         RNP_ERROR_KEY_NOT_FOUND
-    }
+    })
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_signature_get_times(sig: *mut RnpOpVerifySignature,
+fn rnp_op_verify_signature_get_times(sig: *const RnpOpVerifySignature,
                                      created: *mut u32,
                                      expires: *mut u32)
                                      -> RnpResult {
     rnp_function!(rnp_op_verify_signature_get_times, crate::TRACE);
     use std::time::UNIX_EPOCH;
-    assert_ptr!(sig);
+    let sig = assert_ptr_ref!(sig);
+    arg!(created);
+    arg!(expires);
+
     if ! created.is_null() {
         *created = (*sig).sig.signature_creation_time().unwrap_or(UNIX_EPOCH)
             .duration_since(UNIX_EPOCH).unwrap().as_secs()
@@ -393,17 +404,20 @@ fn rnp_op_verify_signature_get_times(sig: *mut RnpOpVerifySignature,
                 as u32
         }).unwrap_or(0);
     }
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_protection_info(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_protection_info(op: *const RnpOpVerify,
                                      mode: *mut *mut c_char,
                                      cipher: *mut *mut c_char,
                                      valid: *mut bool)
                                      -> RnpResult {
     rnp_function!(rnp_op_verify_get_protection_info, crate::TRACE);
     let op = assert_ptr_ref!(op);
+    arg!(mode);
+    arg!(cipher);
+    arg!(valid);
 
     if ! mode.is_null() {
         *mode = str_to_rnp_buffer(op.result.mode.to_rnp_id());
@@ -424,48 +438,49 @@ fn rnp_op_verify_get_protection_info(op: *mut RnpOpVerify,
             false
         };
     }
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_used_recipient(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_used_recipient(op: *const RnpOpVerify,
                                     pkesk: *mut *const PKESK)
                                     -> RnpResult {
     rnp_function!(rnp_op_verify_get_used_recipient, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(pkesk);
+    let pkesk = assert_ptr_mut!(pkesk);
 
     *pkesk = op.result.pkesk_used.as_ref().map(|p| p as *const _)
         .unwrap_or(std::ptr::null());
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_recipient_at(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_recipient_at(op: *const RnpOpVerify,
                                   idx: size_t,
                                   pkesk: *mut *const PKESK)
                                   -> RnpResult {
     rnp_function!(rnp_op_verify_get_recipient_at, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(pkesk);
+    arg!(idx);
+    let pkesk = assert_ptr_mut!(pkesk);
 
-    if let Some(p) = op.result.pkesks.get(idx) {
+    rnp_return_status!(if let Some(p) = op.result.pkesks.get(idx) {
         *pkesk = p as *const _;
         RNP_SUCCESS
     } else {
         RNP_ERROR_BAD_PARAMETERS
-    }
+    })
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_recipient_count(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_recipient_count(op: *const RnpOpVerify,
                                      count: *mut size_t)
                                      -> RnpResult {
     rnp_function!(rnp_op_verify_get_recipient_count, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(count);
+    let count = assert_ptr_mut!(count);
     *count = op.result.pkesks.len();
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
@@ -473,10 +488,10 @@ fn rnp_recipient_get_keyid(recipient: *const PKESK,
                            keyid: *mut *mut c_char)
                            -> RnpResult {
     rnp_function!(rnp_recipient_get_keyid, crate::TRACE);
-    assert_ptr!(recipient);
-    assert_ptr!(keyid);
+    let recipient = assert_ptr_ref!(recipient);
+    let keyid = assert_ptr_mut!(keyid);
     *keyid = str_to_rnp_buffer(format!("{:X}", (*recipient).recipient()));
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
@@ -484,49 +499,51 @@ fn rnp_recipient_get_alg(recipient: *const PKESK,
                          alg: *mut *mut c_char)
                          -> RnpResult {
     rnp_function!(rnp_recipient_get_alg, crate::TRACE);
-    assert_ptr!(recipient);
-    assert_ptr!(alg);
+    let recipient = assert_ptr_ref!(recipient);
+    let alg = assert_ptr_mut!(alg);
     *alg = str_to_rnp_buffer((*recipient).pk_algo().to_rnp_id());
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_used_symenc(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_used_symenc(op: *const RnpOpVerify,
                                  skesk: *mut *const SKESK)
                                  -> RnpResult {
     rnp_function!(rnp_op_verify_get_used_symenc, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(skesk);
+    let skesk = assert_ptr_mut!(skesk);
     *skesk = op.result.skesk_used.as_ref().map(|p| p as *const _)
         .unwrap_or(std::ptr::null());
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_symenc_at(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_symenc_at(op: *const RnpOpVerify,
                                idx: size_t,
                                skesk: *mut *const SKESK)
                                -> RnpResult {
     rnp_function!(rnp_op_verify_get_symenc_at, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(skesk);
-    if let Some(p) = op.result.skesks.get(idx) {
+    arg!(idx);
+    let skesk = assert_ptr_mut!(skesk);
+
+    rnp_return_status!(if let Some(p) = op.result.skesks.get(idx) {
         *skesk = p as *const _;
         RNP_SUCCESS
     } else {
         RNP_ERROR_BAD_PARAMETERS
-    }
+    })
 }
 
 #[no_mangle] pub unsafe extern "C"
-fn rnp_op_verify_get_symenc_count(op: *mut RnpOpVerify,
+fn rnp_op_verify_get_symenc_count(op: *const RnpOpVerify,
                                   count: *mut size_t)
                                   -> RnpResult {
     rnp_function!(rnp_op_verify_get_symenc_count, crate::TRACE);
     let op = assert_ptr_ref!(op);
-    assert_ptr!(count);
+    let count = assert_ptr_mut!(count);
     *count = op.result.skesks.len();
-    RNP_SUCCESS
+    rnp_success!()
 }
 
 struct Helper<'a> {
@@ -555,7 +572,7 @@ impl<'a> VerificationHelper for Helper<'a> {
     }
 
     fn check(&mut self, structure: MessageStructure) -> openpgp::Result<()> {
-        rnp_function!(Helper::check, super::TRACE);
+        rnp_function!(Helper::check, TRACE);
 
         use self::VerificationError::*;
 
@@ -979,8 +996,8 @@ impl<'a> DecryptionHelper for Helper<'a> {
             let keyid = pkesk.recipient();
             t!("trying to decrypt PKESK with recipient {}", keyid);
             if let Some((cert, key)) = self.get_public_key(keyid) {
-                let mut pair = ipc::gnupg::KeyPair::new(
-                    &ipc::gnupg::Context::new()?, key.parts_as_public())?;
+                let mut pair = gnupg::KeyPair::new(
+                    &gnupg::Context::new()?, key.parts_as_public())?;
                 if let Ok(vcert) = cert.with_policy(&*self.ctx.policy(), None) {
                     pair = pair.with_cert(&vcert);
                 }
