@@ -61,6 +61,7 @@ use std::sync::RwLockReadGuard;
 use std::thread;
 use std::time::Duration;
 
+use anyhow::Result;
 use tokio::task::JoinSet;
 
 use sequoia_openpgp as openpgp;
@@ -78,6 +79,9 @@ use rand_distr::{Poisson, Distribution};
 use crate::Keystore;
 use crate::keystore::KeystoreData;
 
+/// Controls tracing in this module.
+const TRACE: bool = crate::TRACE;
+
 // This is a simple heuristic to check whether a certificate might be
 // flooded.  If a User ID or attribute has more than this number of
 // third-party certificates that we prune ones that are not useful.
@@ -94,12 +98,12 @@ impl Parcimonie {
     ///
     /// You can poll for updates using `Parcimonie::updates`.  To
     /// watch additional certificates, use `Parcimonie::monitor`.
-    pub fn new<P>(policy: P, ks: &Keystore) -> Self
+    pub fn new<P>(policy: P, ks: &Keystore) -> Result<Self>
     where P: Policy + 'static
     {
-        ParcimonieServer::new(policy, ks.create_ref());
-        Parcimonie {
-        }
+        ParcimonieServer::new(policy, ks.create_ref())?;
+        Ok(Parcimonie {
+        })
     }
 }
 
@@ -112,20 +116,23 @@ impl<P: Policy + 'static> ParcimonieServer<P> {
     // Instantiates a new parcimonie server, which keeps the key store
     // up to date.
     fn new(policy: P, ks: Arc<RwLock<KeystoreData>>)
+           -> Result<()>
     {
         let pdata = ParcimonieServer {
             ks: ks,
             policy,
         };
 
-        thread::spawn(move || ParcimonieServer::run(pdata));
+        thread::Builder::new().name("sq parcimonie".to_string())
+            .spawn(move || ParcimonieServer::run(pdata))?;
+        Ok(())
     }
 
     fn run(mut self) {
-        rnp_function!(ParcimonieServer::run, super::TRACE);
+        rnp_function!(ParcimonieServer::run, TRACE);
 
         // Sleep for a while to avoid interfering with TB's startup.
-        let delay = if super::TRACE { 30 } else { 5 * 60 };
+        let delay = if TRACE { 30 } else { 5 * 60 };
         thread::sleep(Duration::new(delay, 0));
 
         let rt = tokio::runtime::Runtime::new()
@@ -141,7 +148,7 @@ impl<P: Policy + 'static> ParcimonieServer<P> {
     }
 
     async fn worker(&mut self) -> openpgp::Result<()> {
-        rnp_function!(Parcimonie::woker, super::TRACE);
+        rnp_function!(Parcimonie::woker, TRACE);
 
         let mut rng = rand::thread_rng();
 
@@ -372,6 +379,7 @@ impl<P: Policy + 'static> ParcimonieServer<P> {
                 for cert in certs.into_iter() {
                     ks.insert(cert);
                 }
+                ks.mark_updated();
             }
         }
     }
@@ -385,7 +393,7 @@ impl<P: Policy + 'static> ParcimonieServer<P> {
     ///
     /// This function takes a read lock on the keystore.
     fn clean(&self, cert: Cert) -> Option<Cert> {
-        rnp_function!(Parcimonie::clean, super::TRACE);
+        rnp_function!(Parcimonie::clean, TRACE);
         use std::collections::HashMap;
         use std::collections::hash_map::Entry;
 
